// Weather Note 12.7.20
/* With many thanks to Becky Stern for help with this code. It is
    built on her Weather Station and Affirmation Mirror -- both
    wonderful projects in their own right. So if you like this one,
    check those out as well as her other projects and instructables. -ABK
*/
#include <Wire.h>
#include <SPI.h>
#include <Adafruit_GFX.h>
#include <Adafruit_LEDBackpack.h>
#include "Adafruit_NeoPixel.h"
#include <ESP8266WiFi.h>
#include <AdafruitIO.h>
#include <Adafruit_MQTT.h>
#include <Adafruit_SH110X.h> / for OLED


// Adafruit IO Subscription Example
//
// Adafruit invests time and resources providing this open source code.
// Please support Adafruit and open source hardware by purchasing
// products from Adafruit!
//
// Written by Todd Treece for Adafruit Industries
// Copyright (c) 2016 Adafruit Industries
// Licensed under the MIT license.
//
// All text above must be included in any redistribution.

// edit the config.h tab and enter your Adafruit IO credentials
// and any additional configuration needed for WiFi, cellular,
// or ethernet clients.

#include "config.h"

// Configuration you can optionally change (but probably want to keep the same):
#define MOTION_PIN      14
#define PIXEL_PIN        2                      // Pin connected to the NeoPixel data input.
#define PIXEL_COUNT      12                      // Number of NeoPixels.
#define PIXEL_TYPE      NEO_GRBW + NEO_KHZ800   // Type of the NeoPixels (see strandtest example).

// before running this code, create feeds on Adafruit IO that match these names:
AdafruitIO_Feed *hightemp = io.feed("hightemp"); // set up the 'hightemp' feed Weather Underground by way of IFTTT and Adafruit IO
AdafruitIO_Feed *precipitation = io.feed("precipitation"); // set up the 'precipitation' feed Weather Underground by way of IFTTT and Adafruit IO
AdafruitIO_Feed *textnote = io.feed("textnote"); //set up the 'textnote' feed from Zapier by way of Adafruit IO


Adafruit_7segment matrix = Adafruit_7segment();            // This is the 7 segment display object
Adafruit_AlphaNum4 alpha4 = Adafruit_AlphaNum4();          // This alphanumeric pair of two displays
Adafruit_SH110X display = Adafruit_SH110X(64, 128, &Wire); // This is the OLED display object
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(PIXEL_COUNT, PIXEL_PIN, PIXEL_TYPE); // create NeoPixels object

//== START SETUP FOR 3 ALPHANUMERIC DISPLAYS FOR SCROLLING REMINDER===================//

struct { // label the alphanumeric displays with their i2c addresses
  uint8_t           addr;          // I2C address
  Adafruit_AlphaNum4 alpha4;       // 7segment object
} disp[] = {
  { 0x71, Adafruit_AlphaNum4() },  // left-hand display
  { 0x72, Adafruit_AlphaNum4() },  // middle display
  { 0x74, Adafruit_AlphaNum4() },  // right-hand display
};

//== END SETUP FOR 3 ALPHANUMERIC DISPLAYS FOR SCROLLING REMINDER=====================//

//== PIR sensor state
int current = 0;
int last = 0;

void setup() {

  // start the serial connection

  Serial.begin(115200);
  

  //end OLED//BEGIN 7 SEGMENT MATRIX=======================//

  matrix.begin(0x70); // initialize matrix display for TEMP

  //BEGIN NEOPIXELS========================================//

  // Initialize NeoPixels and turn them off.

  pixels.begin();
  lightPixels(pixels.Color(0, 0, 0, 0));

  //==start trying to connect to Adafruit IO===============//
  Serial.print("Connecting to Adafruit IO");

  // connect to io.adafruit.com
  io.connect();

  // set up a message handler for the count feed.
  // the handleMessage function (defined below)
  // will be called whenever a message is
  // received from adafruit io.
  hightemp->onMessage(handleTemp);
  precipitation->onMessage(handleCondition);
  textnote->onMessage(handleTextNote);


  // wait for a connection
  while (io.status() < AIO_CONNECTED) {
    Serial.print(".");
    delay(500);
  }

  // we are connected
  Serial.println();
  Serial.println(io.statusText());
//============SETUP FORfor OLED===================================//
  Serial.println("128x64 OLED FeatherWing test");
  display.begin(0x3C, true); // Address 0x3C default
  Serial.println("OLED on");
  // Show image buffer on the display hardware.
  // Since the buffer is intialized with an Adafruit splashscreen
  // internally, this will display the splashscreen.
  display.display();
  delay(1000);
  // Clear the buffer.
  display.clearDisplay();
  display.display();
  display.setRotation(1);
  display.setTextSize(2);
  display.setTextColor(SH110X_WHITE);
  display.setCursor(0, 0);
  //== SETUP FOR 3 ALPHANUMERIC DISPLAYS===============================================//
  pinMode(MOTION_PIN, INPUT);
  for (uint8_t i = 0; i < 3; i++) { // Initialize 3 displays

    disp[i].alpha4.begin(disp[i].addr);
    disp[i].alpha4.clear();
    disp[i].alpha4.setBrightness(5);
    disp[i].alpha4.writeDisplay();
  }
  showAlpha4(); // flash display with a message to test it on startup
  delay(500);
  for (uint8_t i = 0; i < 3; i++) { // clear the displays
    disp[i].alpha4.clear();
    disp[i].alpha4.writeDisplay();
  }
}

//== END ALPHANUMERIC===========================================================

void loop() {

  // io.run(); is required for all sketches.
  // it should always be present at the top of your loop
  // function. it keeps the client connected to
  // io.adafruit.com, and processes any incoming data.

  io.run();

  // grab the current state of the button.
  if (digitalRead(MOTION_PIN) == HIGH)
    current = 1;
  else
    current = 0;
  Serial.println(current);
  // return if the value hasn't changed
  if (current == last)
    return; //skip the rest of the program and start the loop over
  //this rest of the program only executes if the sensor state has changed
  if (current == 1) {
    showAlpha4();
    //adjust this delay (below) to help phrase show longer
    delay(500);
  } else {
    for (uint8_t i = 0; i < 3; i++) {
      disp[i].alpha4.clear();
      disp[i].alpha4.writeDisplay();
    }
  }
  last = current;

}

// this function is called whenever a feed message
// is received from Adafruit IO. it was attached to
// the feed in the setup() function above.

//===GRAB THE DAILY FORECASTED HIGH TEMP AND SEND TO 7 SEGMENT DISPLAY =============//

void handleTemp(AdafruitIO_Data *data) {

  Serial.print("received <- ");
  Serial.println(data->value());  // print the temperature data to the serial monitor
  int todaysHigh = data->toInt(); // store the incoming temperature data as an integer
  matrix.print(todaysHigh, DEC);  // send the temperature value to the display
  matrix.setBrightness(5);
  matrix.writeDisplay();          // light up display
  delay(500);                     // wait half a second
}

//==REMEMBER THE MILK NOTE VIA ZAPIER AND SEND TO OLED DISPLAY ==========//

void handleTextNote(AdafruitIO_Data *data) {
  //  Serial.print("forecast for today = ");
  Serial.print("note = ");
  Serial.println(data->value());  // print the weather forcast summary to the serial monitor
  delay(500);                     // wait half a second
  //next will be to clear the display and write to top half of OLED display
  display.clearDisplay(); // this works
  display.display();
  display.setCursor(0, 2);
  display.setTextWrap(true);
  display.print("");
  display.print(data->value());  // print the text out
  delay(10);
  yield();
  display.display();
  delay(500);
}

//==REMEMBER THE MILK NOTE VIA ZAPIER AND SEND TO OLED DISPLAY ======//
//==WEATHER CONDITIONS TO SEND TO NEOPIXEL COMBINATIONS =============//

void handleCondition(AdafruitIO_Data *data) {

  lightPixels(pixels.Color(0, 0, 0, 0)); // reset all pixels to off
  String forecast = data->toString(); // store the incoming weather data in a string

  //the following strings store the varous IFTTT weather report words I've discovered so far
  String rain = String("Rain");
  String lightrain = String("Light Rain");
  String rainshower = String ("Rain Shower");
  String snow = String("Snow");
  String cloudy = String("Cloudy");
  String mostlycloudy = String("Mostly Cloudy");
  String partlycloudy = String("Partly Cloudy");
  String clearsky = String("Clear");
  String fair = String("Fair");
  String sunny = String("Sunny");
  String rainandsnow = String("Rain and Snow");
  String snowshower = String("Snow Shower");

  // These if statements compare the incoming weather variable to the stored conditions, and control the
  // NeoPixels accordingly.

  // SUN pixels 0, 1 and 2 if there's sun in the forecast these are yellow - works
  if (forecast.equalsIgnoreCase(clearsky) || forecast.equalsIgnoreCase(fair) || forecast.equalsIgnoreCase(sunny)) {
    Serial.println("conditions are sunny now");
    pixels.setPixelColor(0, pixels.Color(255, 255, 0, 0));
    pixels.setPixelColor(1, pixels.Color(255, 255, 0, 0));
    pixels.setPixelColor(2, pixels.Color(255, 255, 0, 0));
  }

  // CLOUDY = 3, 4 and 5   if clouds in the forecast pale white blue
  if (forecast.equalsIgnoreCase(cloudy) || forecast.equalsIgnoreCase(mostlycloudy) || forecast.equalsIgnoreCase(partlycloudy)) {
    Serial.println("conditions are cloudy now");
    pixels.setPixelColor(3, pixels.Color(0, 0, 180, 180));
    pixels.setPixelColor(4, pixels.Color(0, 0, 180, 180));
    pixels.setPixelColor(5, pixels.Color(0, 0, 180, 180));
  }


  // SNOW pixels 9,10, and 11   if there's snow in the forecast, these are basically white
  if (forecast.equalsIgnoreCase(snow) || forecast.equalsIgnoreCase(rainandsnow) || forecast.equalsIgnoreCase(snowshower)) {
    Serial.println("it is snowing now");
    pixels.setPixelColor(3, pixels.Color(0, 0, 180, 180));
    pixels.setPixelColor(4, pixels.Color(0, 0, 180, 180));
    pixels.setPixelColor(5, pixels.Color(0, 0, 180, 180));
    pixels.setPixelColor(6, pixels.Color(255, 10, 0, 200));
    pixels.setPixelColor(7, pixels.Color(255, 10, 0, 200));
    pixels.setPixelColor(8, pixels.Color(255, 10, 0, 200));
  }
  // RAINY pixels 6,7 and 8 if there's rain in the forecast these are blue
  if (forecast.equalsIgnoreCase(rain) || forecast.equalsIgnoreCase(lightrain) || forecast.equalsIgnoreCase(rainshower)) {
    Serial.println("it is raining now");
    pixels.setPixelColor(3, pixels.Color(0, 0, 180, 180));
    pixels.setPixelColor(4, pixels.Color(0, 0, 180, 180));
    pixels.setPixelColor(5, pixels.Color(0, 0, 180, 180));
    pixels.setPixelColor(9, pixels.Color(0, 200, 255, 0));
    pixels.setPixelColor(10, pixels.Color(0, 200, 255, 0));
    pixels.setPixelColor(11, pixels.Color(0, 200, 255, 0));
  }
  pixels.show(); // light up the pixels
}

// Function to set all the NeoPixels to the specified color.
void lightPixels(uint32_t color) {
  for (int i = 0; i < PIXEL_COUNT; ++i) {
    pixels.setPixelColor(i, color);
  }
  pixels.show();
}

//==THE REST OF ALPHANUMERIC DISPLAY CODE TO DISPLAY SCROLLING MESSAGE ====================>

void showAlpha4() {
  disp[0].alpha4.writeDigitAscii(0, '*');
  disp[0].alpha4.writeDigitAscii(1, 'M');
  disp[0].alpha4.writeDigitAscii(2, 'A');
  disp[0].alpha4.writeDigitAscii(3, 'S');
  disp[1].alpha4.writeDigitAscii(0, 'K');
  disp[1].alpha4.writeDigitAscii(1, ' ');
  disp[1].alpha4.writeDigitAscii(2, '+');
  disp[1].alpha4.writeDigitAscii(3, ' ');
  disp[2].alpha4.writeDigitAscii(0, 'K');
  disp[2].alpha4.writeDigitAscii(1, 'E');
  disp[2].alpha4.writeDigitAscii(2, 'Y');
  disp[2].alpha4.writeDigitAscii(3, 'S');
  disp[0].alpha4.writeDisplay();
  disp[1].alpha4.writeDisplay();
  disp[2].alpha4.writeDisplay();
}
